
import requests
from .parsers import Parser
from .exceptions import JWTError, PermissionsError, QueryError
from .Calls import Azure, BaseStation, Messages, Models, Mqtt, Nodes, Plugins, System


class Calls(Azure, BaseStation, Messages, Models, Mqtt, Nodes, Plugins, System):
    """
    Is a class for connecting to and accessing messages on BehrTech's industrial gateways
    """

    def __init__(self, username: str, password: str, server_address: str):
        Azure.__init__(self)
        BaseStation.__init__(self)
        Messages.__init__(self)
        Models.__init__(self)
        Mqtt.__init__(self)
        Nodes.__init__(self)
        Plugins.__init__(self)
        System.__init__(self)
        self.username = username
        self.password = password
        self.server_address = server_address
        self.jwt_token = self.login()

    def __repr__(self):
        return f"Connection to {self.server_address} for user {self.username}"

    def set_username(self, username: str):
        """
        Sets the login username for the given gateway

        :param username: username credential
        """

        self.username = username

    def get_username(self) -> str:
        """
        Returns the username credential for the given gateway

        :return: The username credential for the given gateway
        """

        return self.username

    def set_password(self, password: str):
        """
        Sets the login password for the given gateway

        :param password: password credential
        """

        self.password = password

    def get_password(self) -> str:
        """
        Returns the password credential for the given gateway

        :return: The password credential for the given gateway
        """

        return self.password

    def set_server_address(self, server_address):
        """
        Sets the IP address for the given gateway

        :param server_address: The IP address of the given gateway
        """

        self.server_address = server_address

    def get_server_address(self) -> str:
        """
        Returns the set IP address of the gateway

        :return: The set IP address of the gateway
        """

        return self.server_address

    def test_connection(self) -> bool:
        if self.jwt_token:
            return True
        else:
            return False

    def get_all_nodes(self, return_count: int, offset: int) -> Parser:
        """
        Returns data on all connected nodes from the gateway.

        :param return_count: Is the amount of nodes to be requested
        :param offset: Is the node count to start the node data request from
        :return: the requested node data from the gateway
        """

        message = self.server_address + f"/v2/nodes?returnCount={return_count}&offset={offset}"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_node(self, ep_eui: str) -> Parser:
        """
        Returns data on a requested node from the gateway.

        :param ep_eui: Is the unique identifier of the node to be requested
        :return: the requested node data from the gateway
        """

        message = self.server_address + f"/v2/nodes/{ep_eui}"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_MQTT_mappings(self, return_count: int, offset: int) -> Parser:
        """
        Returns data on all setup MQTT Topics from the gateway.

        :param return_count: Is the amount of MQTT Topics to be requested
        :param offset: Is the MQTT Topic count to start the MQTT Topic data request from
        :return: the requested MQTT Topic data from the gateway
        """

        message = self.server_address + f"/v2/mqttmapping?returnCount={return_count}&offset={offset}"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_all_MQTT_brokers(self, return_count: int, offset: int) -> Parser:
        """
        Returns data on all setup MQTT Brokers from the gateway.

        :param return_count: Is the amount of MQTT Brokers to be requested
        :param offset: Is the MQTT Broker count to start the MQTT Broker data request from
        :return: the requested MQTT Broker data from the gateway
        """

        message = self.server_address + f"/v2/broker?returnCount={return_count}&offset={offset}"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_MQTT_broker(self, broker_id: str) -> Parser:
        """
        Returns data on a requested MQTT Broker from the gateway.

        :param broker_id: Is the unique identifier of the MQTT Broker to be requested
        :return: the requested MQTT Broker data from the gateway
        """

        message = self.server_address + f"/v2/broker/{broker_id}"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_auth_ticker(self) -> Parser:
        """
        Returns data on auth ticketers.

        :return: the requested MQTT Topic data from the gateway
        """

        message = self.server_address + "/v2/auth/ticket"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_gateway_data(self) -> Parser:
        """
        Returns data on the configured gateways.

        :return: the requested gateway data
        """

        message = self.server_address + "/v2/system"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))

    def get_azure_mapping(self) -> Parser:
        """
        Returns data for setup Azure connections on the gateway.

        :return: the requested Azure connection data
        """

        message = self.server_address + "/v2/azuremapping"

        return Parser(requests.get(message, headers={"Authorization": f"Bearer {self.jwt_token}"}))





class PyBehrtechException(Exception):
    """Base exception used by this module."""
    pass


class JWTError(PyBehrtechException):
    """A request for the JWT failed"""

    def __init__(self, message):
        PyBehrtechException.__init__(self, message)


class PermissionsError(PyBehrtechException):
    """User of request doesn't have permissions"""

    def __init__(self, message):
        PyBehrtechException.__init__(self, message)


class QueryError(PyBehrtechException):
    """Request to endpoint is invalid or was built incorrectly"""

    def __init__(self, url, message):
        self.url = url
        PyBehrtechException.__init__(self, url, message)




import json
from datetime import datetime
from typing import Union


class JSONParser:

    def __init__(self):
        self.data = None

    def get_rxData_messages(self) -> str:
        """
        Sources all data of the class and returns only rxData messages as a json string

        :return: All sensor data for each message as a json string
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        remove = [x for x in messages if x.get("command") != "rxData"]

        for rem in remove:
            messages.remove(rem)

        data['count'] = data['count'] - len(remove)
        data['total'] = data['total'] - len(remove)
        data['messages'] = messages

        return json.dumps(data, indent=4, sort_keys=True)

    def get_attPrpAcc_messages(self) -> str:
        """
        Sources all data of the class and returns only attPrpAcc messages as a json string

        :return: All attPrpAcc messages as a json string
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        remove = [x for x in messages if x.get("command") != "attPrpAcc"]

        for rem in remove:
            messages.remove(rem)

        data['count'] = data['count'] - len(remove)
        data['total'] = data['total'] - len(remove)
        data['messages'] = messages

        return json.dumps(data, indent=4, sort_keys=True)

    def get_detPrpAcc_messages(self) -> str:
        """
        Sources all data of the class and returns only detPrpAcc messages as a json string

        :return: All detPrpAcc messages as a json string
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        remove = [x for x in messages if x.get("command") != "detPrpAcc"]

        for rem in remove:
            messages.remove(rem)

        data['count'] = data['count'] - len(remove)
        data['total'] = data['total'] - len(remove)
        data['messages'] = messages

        return json.dumps(data, indent=4, sort_keys=True)

    def get_message_for_id(self, id_number) -> str:
        """
        Sources all the data of the class to retrieve a message for the provided id

        :param id_number: The id number of the message that is being requested
        :return: The message as a json string
        """

        data = json.loads(self.data)

        for x in data.get("messages"):
            if x.get("_id") == id_number:

                data['count'] = 1
                data['total'] = 1
                data['messages'] = x

                return json.dumps(data, indent=4, sort_keys=True)

    def get_messages_in_date_range(self, start: datetime, end: datetime) -> str:
        """
        Sources all the data of the class to retrieve all messages that are within the specified date range

        :param start: Lowest date of date range
        :param end: Highest date of date range
        :return: Messages within the specified date range as a list
        """

        user_data = []
        data = json.loads(self.data)

        for x in data.get("messages"):
            if x.get("time"):
                time_split = x.get("time").strip("Z").split("T")
                message_time = datetime.strptime(f"{time_split[0]} {time_split[1]}", '%Y-%m-%d %H:%M:%S')

                if start <= message_time <= end:
                    user_data.append(x)

                elif message_time < start:
                    data['count'] = data['total'] = len(user_data)
                    data['messages'] = user_data

                    return json.dumps(data, indent=4, sort_keys=True)

    def get_messages_to_id(self, id_num: str) -> Union[str, bool]:
        """
        Sources all the data of the class to retrieve all messages up to the message with the provided ID

        :param id_num: The ID number of the message to requested messages to
        :return: Messages from the start to the message with the provided id or False if id isn't found
        """

        user_data = []
        data = json.loads(self.data)

        for x in data.get("messages"):

            if x.get('_id') == id_num:
                data['count'] = data['total'] = len(user_data)
                data['messages'] = user_data

                return json.dumps(data, indent=4, sort_keys=True)

            user_data.append(x)

        return False




from py_behrtech.Calls.azure import Azure
from py_behrtech.Calls.base_station import BaseStation
from py_behrtech.Calls.messages import Messages
from py_behrtech.Calls.models import Models
from py_behrtech.Calls.mqtt import Mqtt
from py_behrtech.Calls.nodes import Nodes
from py_behrtech.Calls.plugins import Plugins
from py_behrtech.Calls.system import System




import requests

from py_behrtech.parsers import Parser
from py_behrtech.exceptions import JWTError, PermissionsError, QueryError


class Messages:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None
        self.req = None

    @property
    def check_status_code(self):
        if self.req.status_code == 400:
            raise QueryError(url=self.req.url, message="Endpoint is invalid or was built incorrectly")
        elif self.req.status_code == 401:
            raise JWTError(message="JWT Access token is missing or invalid")
        elif self.req.status_code == 403:
            raise PermissionsError(message="User doesn't have the correct permissions to access this data")
        elif self.req.status_code == 404:
            raise QueryError(url=self.req.url, message="Endpoint is invalid or was built incorrectly")

    def messages_delete(self, deleteAll: bool = False):
        """
        Deletes all messages from the gateways

        :param deleteAll: Verifies deletion was on purpose
        :return:
        """

        self.req = requests.delete(url=self.server_address + f"/v2/messages", params={'deleteAll': deleteAll},
                                   headers={"Authorization": f"Bearer {self.jwt_token}"})

        if self.req.status_code == 200:
            return Parser(req=self.req)
        else:
            self.check_status_code()

    def messages_get(self, return_count: int = '', offset: int = '', epEui: str = '', epName: str = '', bsEui: str = '',
                     sensorType: str = '') -> Parser:
        """
        Returns messages from the gateway.

        :param return_count: The amount of messages to be requested. (-1 for all)
        :param offset: Message number to start the request from
        :param epEui: The epEui number of the sensor to be requested
        :param epName: Endpoint of which messages to be requested. Name is not unique
        :param bsEui: The bsEui number of the messages to be requested
        :param sensorType: The node type of a sensor to be requested
        :return: Parser object
        """

        parameters = ''

        if return_count:
            parameters += f"?returnCount={return_count}"
        if offset:
            parameters += (f"&offset={offset}" if parameters else f"?offset={offset}")
        if epEui:
            parameters += f"&epEui={epEui}" if parameters else f"?epEui={epEui}"
        if epName:
            parameters += f"&epName={epName}" if parameters else f"?epName={epName}"
        if bsEui:
            parameters += f"&bsEui={bsEui}" if parameters else f"?bsEui={bsEui}"
        if sensorType:
            parameters += f"&sensorType={sensorType}" if parameters else f"?sensorType={sensorType}"

        self.req = requests.get(url=self.server_address + f"/v2/messages" + parameters,
                                headers={"Authorization": f"Bearer {self.jwt_token}"})

        if self.req.status_code == 200:
            return Parser(req=self.req)
        else:
            self.check_status_code()

    def messages_post(self):
        # TODO: build this function
        pass




import requests
from requests.exceptions import RequestException
from urllib3.exceptions import MaxRetryError, ConnectTimeoutError

from py_behrtech.exceptions import JWTError


class System:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None

    def login(self) -> str:
        """
        Gets the JWT token of the login endpoint.
        """
        try:
            res = requests.post(
                self.server_address + "/v2/login",
                json={'user': self.username, 'password': self.password},
                timeout=3
            )

            if res.status_code == 200:
                # Successfully logged in
                return res.json().get("JWT")
            elif res.status_code == 401:
                # Credentials provided are incorrect
                raise JWTError(message="The wrong server IP address or login credentials were provided")

        except (TimeoutError, ConnectTimeoutError, MaxRetryError, RequestException):
            raise JWTError(message="The wrong server IP address or login credentials were provided")




import requests

from py_behrtech.parsers import Parser
from py_behrtech.exceptions import JWTError, PermissionsError, QueryError


class Models:

    def __init__(self):
        self.username = None
        self.password = None
        self.server_address = None
        self.jwt_token = None
        self.req = None

    @property
    def check_status_code(self):
        if self.req.status_code == 400:
            raise QueryError(url=self.req.url, message="Endpoint is invalid or was built incorrectly")
        elif self.req.status_code == 401:
            raise JWTError(message="JWT Access token is missing or invalid")
        elif self.req.status_code == 403:
            raise PermissionsError(message="User doesn't have the correct permissions to access this data")
        elif self.req.status_code == 404:
            raise QueryError(url=self.req.url, message="Endpoint is invalid or was built incorrectly")

    def models_delete(self):
        pass

    def models_get(self, return_count: int = '', offset: int = ''):
        """
        Gets information on all registered node models from the gateway.

        :return: Requested data for all node models from the gateway
        """

        parameters = ''

        if return_count:
            parameters += f"?returnCount={return_count}"
        if offset:
            parameters += (f"&offset={offset}" if parameters else f"?offset={offset}")

        self.req = requests.get(url=self.server_address + f"/v2/sensormodels" + parameters,
                                headers={"Authorization": f"Bearer {self.jwt_token}"})

        if self.req.status_code == 200:
            return Parser(req=self.req)
        else:
            self.check_status_code()

    def models_post(self):
        pass

    def models_type_delete(self):
        pass

    def models_type_get(self, model_type: str):
        """
        Gets information on a requested registered node model from the gateway

        :param model_type: Unique identifier of the node model to be requested
        :return: Requested node model data from the gateway
        """

        self.req = requests.get(url=self.server_address + f"/v2/sensormodels/{model_type}",
                                headers={"Authorization": f"Bearer {self.jwt_token}"})

        if self.req.status_code == 200:
            return Parser(req=self.req)
        else:
            self.check_status_code()

    def models_type_post(self):
        pass



import setuptools

with open("py_behrtech/README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py_behrtech",
    version="2020.11.03.2",
    license="MIT",
    author="Matthew Ashley",
    author_email="matthewashley@verntechnologies.com",
    description="A package to interface with BehrTech's gateway API for requesting and parsing data",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Vern-Technologies/py_behrtech",
    packages=setuptools.find_packages(),
    install_requires=[
        'requests',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)




import os
import json
from requests import Response
from datetime import datetime

from .json_parsers import JSONParser


class Parser(JSONParser):
    """
    Is the base class for breaking data returned from a gateway computer from BehrTech into usable components. Provides
    stander functions that apply to all sensor types connected to a gateway.
    """

    def __init__(self, req: Response):
        JSONParser.__init__(self)
        self.response: Response = req
        self.data: str = req.text

    def __repr__(self):
        return f"Parser object for request at {self.response.url} with status code {self.response.status_code}"

    def set_response(self, response: Response):
        """
        Sets the response of the class to the passed in requests Response. Also resets the data of the class from the
        text of the passed in Response.

        :param response: request Response
        """

        self.response = response
        self.data = response.text

    def get_response(self):
        """
        Returns the response of the class

        :return: The response of the class
        """

        return self.response

    def get_data(self):
        """
        Returns the data of the class

        :return: The data of the class
        """
        return self.data

    def write_data_to_file(self, file_name: str, file_path: os.path = 'Outputs/'):
        """
        Outputs the data of the class to a JSON file

        :param file_path: The path to write the file to
        :param file_name: The name of the file for the data of the class to be outputted to
        """

        file_path = file_path if os.path.isdir(file_path) else 'Outputs/'

        text_file = open(file_path + file_name + ".json", "w")
        text_file.write(self.data)
        text_file.close()

    def get_message_count(self) -> int:
        """
        Returns the total count of messages

        :return: The message count total
        """

        return json.loads(self.data).get("count")

    def get_component_data(self, component: str):
        """
        Sources all the data of the class to retrieve the data for a specified component for each message

        :return: The data for a specified component for each message as a list
        """

        component_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("userDataJSON")

                if check:
                    data = json.loads(check)

                    if "message" not in data.keys() and component in data.keys():
                        component_data.append(data[component])

        return component_data

    def get_message_data(self) -> list:
        """
        Sources all the data of the class to retrieve all sensor data for each message including a time stamp and
        sensor ID

        :return: All sensor data for each message as a list
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("userDataJSON")

                if check:
                    data = json.loads(check)

                    if "message" not in data.keys():
                        data["Time"] = x.get("time")
                        data["Command"] = x.get("command")
                        data["Type"] = x.get("type")
                        data["SensorID"] = x.get("epEui")
                        user_data.append(data)

        return user_data

    def get_all_message_types(self) -> list:
        """
        Sources all the data of the class to retrieve all messages for each message type

        :return: All messages for each message type as a list
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:

            check = x.get("userDataJSON")

            if check:
                data = json.loads(check)
                data["Time"] = x.get("time")
                data["Command"] = x.get("command")
                data["Type"] = x.get("type")
                data["SensorID"] = x.get("epEui")
                data["ID"] = x.get('_id')

            else:
                data = {"Time": x.get("time"), "Command": x.get("command"), "Type": x.get("type"),
                        "SensorID": x.get("epEui"), "ID": x.get('_id')}

            user_data.append(data)

        return user_data

    def get_message_datetime_as_str(self) -> list:
        """
        Sources all the data of the class to retrieve the time stamp for each message

        :return: The time stamp of each message as a list broken into individual dicts and separated as date and time
        """

        time_info = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                time = x.get("time")

                if time:
                    time_split = time.strip("Z").split("T")
                    time_info.append({"date": time_split[0], "time": time_split[1]})

        return time_info

    def get_message_datetime_as_datetime(self) -> list:
        """
        Sources all the data of the class to retrieve the time stamp for each message

        :return: The time stamp of each message as a list formatted as a datetime
        """

        time_info = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                time = x.get("time")

                if time:
                    time_split = time.strip("Z").split("T")
                    new_time = f"{time_split[0]} {time_split[1]}"
                    time_info.append(datetime.strptime(new_time, '%Y-%m-%d %H:%M:%S'))

        return time_info

    def get_message_data_in_number_range(self, count: int, offset: int) -> list:
        """
        Sources all the data of the class to retrieve the amount of messages requested at the desired offset

        :param count: The amount of messages to be returned
        :param offset: The message position to start from
        :return: A list of the message data for the messages requested
        """

        amount = 0
        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for index, x in enumerate(messages):
            if index >= offset:
                if amount != count:
                    if x.get("command") == "rxData":

                        check = x.get("userDataJSON")

                        if check:
                            data = json.loads(check)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                user_data.append(data)
                                amount += 1

                else:
                    break

        return user_data

    def get_message_data_in_date_range(self, start: datetime, end: datetime) -> list:
        """
        Sources all the data of the class to retrieve all sensor data for a message including a time stamp and
        sensor ID that is within the specified date range

        :param start: Lowest date of date range
        :param end: Highest date of date range
        :return: All sensor data for each message as a list
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                time = x.get("time")

                if time:
                    time_split = time.strip("Z").split("T")
                    new_time = f"{time_split[0]} {time_split[1]}"

                    message_time = datetime.strptime(new_time, '%Y-%m-%d %H:%M:%S')

                    if start <= message_time <= end:
                        check = x.get("userDataJSON")

                        if check:
                            data = json.loads(check)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                user_data.append(data)

                    elif message_time < start:
                        break

        return user_data

    def get_date_ranges(self) -> dict:
        """
        Sources all the data of the class to retrieve the date range of the data of the class

        :return: The date range of the data of the class as a dict
        """

        time = self.get_message_datetime_as_str()

        return {"Recent": time[0]["date"], "Oldest": time[-1]["date"]}

    def get_message_ids(self) -> list:
        """
        Sources all the data of the class to retrieve the id for each message

        :return: The id of each message as a list
        """

        ids = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("_id")

                if check:
                    ids.append(check)

        return ids

    def get_message_data_for_id(self, id_number) -> dict:
        """
        Sources all the data of the class to retrieve all sensor data for a message including a time stamp and
        sensor ID that matches the requested id number

        :param id_number: The id number of the message that data is being requested for
        :return: The data of the message requested for as a dictionary
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check_id = x.get("_id")

                if check_id:
                    if check_id == id_number:

                        check_data = x.get("userDataJSON")

                        if check_data:
                            data = json.loads(check_data)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                return data

    def get_message_position_for_id(self, id_number) -> int:
        """
        Sources all the data of the class to retrieve the position of the message that matches the requested id number

        :param id_number: The id number of the message that data is being requested for
        :return: The position of the messages that matches the requested id number
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        for index, x in enumerate(messages):

            check_id = x.get("_id")

            if check_id:
                if check_id == id_number:
                    return index + 1

    def get_message_epEui(self) -> list:
        """
        Sources all the data of the class to retrieve the epEui for each message

        :return: The epEui of each message as a list
        """

        ids = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check = x.get("epEui")

                if check:
                    ids.append(check)

        return ids

    def get_message_data_for_epEui(self, eui_number) -> list:
        """
        Sources all the data of the class to retrieve all sensor data for a message including a time stamp and
        sensor ID that matches the requested epEui number

        :param eui_number: The epEui number of the message that data is being requested for
        :return: The data of the message requested for as a dictionary
        """

        user_data = []
        data = json.loads(self.data)
        messages = data.get("messages")

        for x in messages:
            if x.get("command") == "rxData":

                check_eui = x.get("epEui")

                if check_eui:
                    if check_eui == eui_number:

                        check_data = x.get("userDataJSON")

                        if check_data:
                            data = json.loads(check_data)

                            if "message" not in data.keys():
                                data["Time"] = x.get("time")
                                data["Command"] = x.get("command")
                                data["Type"] = x.get("type")
                                data["SensorID"] = x.get("epEui")
                                user_data.append(data)

        return user_data

    def get_message_position_for_epEui(self, eui_number) -> int:
        """
        Sources all the data of the class to retrieve the position of the message that matches the requested epEui
        number

        :param eui_number: The epEui number of the message that data is being requested for
        :return: The position of the messages that matches the requested epEui number
        """

        data = json.loads(self.data)
        messages = data.get("messages")

        for index, x in enumerate(messages):

            check_eui = x.get("epEui")

            if check_eui:
                if check_eui == eui_number:
                    return index + 1

    # TODO: rewrite functions

    # def get_components(self, ep_eui: str) -> list:
    #     """
    #     Returns a list of measured components for the specified sensor
    #
    #     :param ep_eui:  The unique identifier of a node
    #     :return: List of measured components
    #     """
    #
    #     sens_type = json.loads(self.get_node(ep_eui=ep_eui).text).get('type')
    #     components = json.loads(self.get_node_model(model_type=sens_type).text).get('components')
    #
    #     return [x for x in components]
    #
    # def get_units(self, ep_eui: str) -> dict:
    #     """
    #     Returns a dict of units for measured components for the specified sensor
    #
    #     :param ep_eui: The unique identifier of a node
    #     :return: Dict of units for measured components
    #     """
    #
    #     sens_type = json.loads(self.get_node(ep_eui=ep_eui).text).get('type')
    #     components = json.loads(self.get_node_model(model_type=sens_type).text).get('components')
    #
    #     return {x: components.get(x)['unit'] for x in components}
